/*
 *****************************************************************************
 *  Program: SFC_readhdf.c
 *  
 *  Purpose: To provide examples of how to read data from the SFC HDF file.
 *  
 *  There are 3 examples of how to read data from the Vdatas on the
 *  SFC HDF file.
 *
 *  Example 1) Illustrates reading string data from a Vdata field and 
 *             storing the data in an array. 
 *  Example 2) Illustrates reading integer data from a Vdata field.
 *  Example 3) Illustrates reading multiple VDatas and how the VDatas in
 *             a given SFC file are related.
 *
 *
 *  An alternate way of extracting the data from an HDF file is through the 
 *  "hdp" utility which is a part of the HDF libraries.  The "hdp" utility
 *  will give information as to the size, type, order, and names of the 
 *  HDF data structures in addition to the data itself.  Some examples are:
 *
 *        "hdp -H"      - provides "help" on how to use the "hdp" utility
 *        "hdp dumpvd"  - lists the options and format for viewing Vdatas
 *
 *  The examples require specification of the 'type' of data to be read
 *  passed as a parameter to the HDF routine. The HDF data types are
 *  defined in the following table. The names (first column) or numeric
 *  values (center column) are valid parameter values; either can be
 *  used in C programs since the names are defined with the appropriate
 *  value in an HDF include file.  
 *
 *                         HDF Data Types
 *
 *  Definition Name   |   Definition Value   |       Description
 *  ------------------|----------------------|--------------------------------
 *  DFNT_CHAR8        |          4           |   8-bit character type
 *  DFNT_CHAR         |          4           |   Same as DFNT_CHAR8
 *  DFNT_UCHAR8       |          3           |   8-bit unsigned character type
 *  DFNT_UCHAR        |          3           |   Same as DFNT_UCHAR8
 *  DFNT_INT8         |         20           |   8-bit integer type
 *  DFNT_UINT8        |         21           |   8-bit unsigned integer type
 *  DFNT_INT16        |         22           |   16-bit integer type
 *  DFNT_UINT16       |         23           |   16-bit unsigned integer type
 *  DFNT_INT32        |         24           |   32-bit integer type
 *  DFNT_UINT32       |         25           |   32-bit unsigned integer type
 *  DFNT_INT64        |         26           |   64-bit integer type
 *  DFNT_UINT64       |         27           |   64-bit unsigned integer type
 *  DFNT_FLOAT32      |          5           |   32-bit floating-point type
 *  DFNT_FLOAT64      |          6           |   64-bit floating-point type
 *
 *  To modify the code to extract different VDatas than what are in the examples, 
 *  the following variables must be changed:
 *   HDF_File    - SFC HDF file name - modify this if a different SFC HDF file is
 *                 being read
 *   sds_name  - sds name - see the Data Products Catalog ("SFC_dpc_r3.ps")
 *                 for valid VData names in the SFC product
 *   sds_rank  - dimension - see the Data Products Catalog ("SFC_dpc_r3.ps")
 *                 for valid VData Field names in the SFC product.
*/

   #include <stdio.h>
   #include <stdlib.h>
   #include <string.h>
   #include <math.h>
   #include "SFC_HDFread_daacv3.h"

/*
 *******************************************************************************
 *  MAIN PROGRAM
 *******************************************************************************
*/

   #define MAX_RECORDS 5  

   void main(int argc, char *argv[])
   {

     char *HDF_File;
     char *sds_name;
     char *vdata_name;
     char *vfield_name;
     int32 sds_rank;
     int32 sds_type;
     int32 sds_status;
     int32 sds_start[MAX_SDS_DIMS];
     int32 sds_incr[MAX_SDS_DIMS];
     int32 sds_edge[MAX_SDS_DIMS];
     int32 num_fields;
     int32 rd_num_fields;
     int32 rd_num_recs;
     int32 vfield_types[MAX_VDATA_FIELDS];
     int32 vfield_order[MAX_VDATA_FIELDS];
     int32 vfield_status;
     long  vdata_status;   
     int32 i;
     void *vdata_data[MAX_VDATA_FIELDS]; 

/*
 *******************************************************************************
 *  Variables assigned for Examples 1, 2 and 3
 *******************************************************************************
*/

     float32 Reg_data[5];
     float32 Hr_data[5];
     float32 SW_data[5][3];
     char    platform[32];

/* 
 ===========================================================================
   Display the software header/banner.
 ============================================================================
*/

     printf ("\n*********************************************************\n");
     printf ("*                SFC Read Software                        *\n");
     printf ("*                                                         *\n");
     printf ("* Version: 2.0                                            *\n");
     printf ("*                                                         *\n");
     printf ("* Date:    May, 2002                                      *\n");
     printf ("*                                                         *\n");
     printf ("* Contact: Atmospheric Sciences Data Center               *\n");
     printf ("*          Science, Users, and Data Services Office       *\n");
     printf ("*          Mail Stop 157D                                 *\n");
     printf ("*          2 South Wright Street                          *\n");
     printf ("*          NASA Langley Research Center                   *\n");
     printf ("*          Hampton, Virginia 23681-2199                   *\n");
     printf ("*          U.S.A.                                         *\n");
     printf ("*                                                         *\n");
     printf ("*          E-mail:   larc@eos.nasa.gov                    *\n");
     printf ("*          Phone:    (757)864-8656                        *\n");
     printf ("*          FAX:      (757)864-8807                        *\n");
     printf ("*                                                         *\n");
     printf ("***********************************************************\n");

/*
 *******************************************************************************
 *  Assign the SFC test HDF file to be read.  If another HDF file is to
 *  be read, enter its name here.
 *******************************************************************************
*/

   HDF_File = "CER_SFC_TRMM-PFM-VIRS_SampleReadFile_999999.199802Z09";

     printf("\nName of SFC HDF file to be read:\n< %s\n",HDF_File);

     if (!Hishdf(HDF_File))
     {
       fprintf(stderr, "ERROR - specified file is not an HDF file.\n");
       exit (1);
     } 

/*
 ===========================================================================
 Example 1) Extract a SDS from the HDF file and store in memory.

  The SDS "region number " will be extracted
  and stored in the array Reg_Box. This example illustrates the use
  of function 'read_sds'.

  'read_sds'
     Function - Reads a SDS data set from a HDF file.
     Input Variables:
        HDF_File - character pointer to the name of the HDF file.
        sds_name - character pointer to the name of the SDS data set.
        sds_rank - integer defining rank of the SDS data set.
        sds_type - integer defining type of SDS data set (see table above).
        sds_strt - integer array of starting index of each dimension of the SDS.
        sds_incr - integer array of increment of each dimension of the SDS.
        sds_edge - integer array of number of values of each dimension
                   of the SDS.
     Output Variable:
        Reg_data   - integer variable storing Region number.

 ============================================================================
*/

     sds_name = "Region Number";
     sds_rank = 3;
     sds_type = 5;
     sds_start[0] = 0;
     sds_start[1] = 0;
     sds_start[2] = 0;
     sds_incr[0] = 1;
     sds_incr[1] = 1;
     sds_incr[2] = 1;
     sds_edge[0] = 5;
     sds_edge[1] = 1;
     sds_edge[2] = 1;
    
     sds_status = read_sds(HDF_File, sds_name, sds_rank, sds_type,
                           sds_start, sds_incr, sds_edge, Reg_data);

     printf ("\n\n*********************************************************\n");
     printf ("*    Example 1:  SDS                                    *\n");
     printf ("*********************************************************\n\n");

     printf ("Region Number  %f\n\n",Reg_data[0]);

/*
 ===========================================================================
 Example 2) Extract a SDS from the HDF file and store in memory.

  The SDS "Hour Box Number " will be extracted
  and stored in the array Reg_Box. This example illustrates the use
  of function 'read_sds'.

  'read_sds'
     Function - Reads a SDS data set from a HDF file.
     Input Variables:
        HDF_File - character pointer to the name of the HDF file.
        sds_name - character pointer to the name of the SDS data set.
        sds_rank - integer defining rank of the SDS data set.
        sds_type - integer defining type of SDS data set (see table above).
        sds_strt - integer array of starting index of each dimension of the SDS.
        sds_incr - integer array of increment of each dimension of the SDS.
        sds_edge - integer array of number of values of each dimension
                   of the SDS.
     Output Variable:
        Hr_data   - integer variable storing Hour Box Number.

 ============================================================================
*/

     sds_name = "Hour Box Number";
     sds_rank = 3;
     sds_type = 5;
     sds_start[0] = 0;
     sds_start[1] = 0;
     sds_start[2] = 0;
     sds_incr[0] = 1;
     sds_incr[1] = 1;
     sds_incr[2] = 1;
     sds_edge[0] = 5;
     sds_edge[1] = 1;
     sds_edge[2] = 1;

     sds_status = read_sds(HDF_File, sds_name, sds_rank, sds_type,
                           sds_start, sds_incr, sds_edge, Hr_data);

     printf ("\n\n*********************************************************\n");
     printf ("*    Example 2:  SDS                                    *\n");
     printf ("*********************************************************\n");

     for (i=0; i < 5; ++i) {
       if (i % 20 == 0) {
         fprintf(stdout,"\n\nREC#\tHourBox #\n");
       }
       fprintf (stdout, "%4.4d:\t%8d\n",
                i+1, (int) Hr_data[i]);
     }    

/*
 ===========================================================================
 Example 3) Extract a SDS from the HDF file and print out data in ascii text

  The SDS "CERES SW TOA Total-Sky" will be extracted
  and written out to a file. This example illustrates the use of functions
  'set_sds_data' and 'read_sds'.

  'set_sds_data'
     Function - Returns a void pointer to the address to write the SDS data set.
     Input Variables:
        sds_rank - integer defining rank of the SDS data set.
        sds_type - integer defining type of SDS data set (see table above).
        sds_dims - integer array of rank 'sds_rank' containing dimensions
                   of the SDS data set.

  'read_sds'
     Function - Reads a SDS data set from a HDF file.
     Input Variables:
        HDF_File - character pointer to the name of the HDF file.
        sds_name - character pointer to the name of the SDS data set.
        sds_rank - integer defining rank of the SDS data set.
        sds_type - integer defining type of SDS data set (see table above).
        sds_strt - integer array of starting index of each dimension of the SDS.
        sds_incr - integer array of increment of each dimension of the SDS.
        sds_edge - integer array of number of values of each dimension
                   of the SDS.
     Output Variable:
        SW_data - output void pointer later cast to type 'sds_type'
 ============================================================================
*/
 
     sds_name = "SW TOA Total-Sky";
     sds_rank = 4;
     sds_type = 5;
     sds_start[0] = 0;
     sds_start[1] = 0;
     sds_start[2] = 0;
     sds_start[3] = 0;
     sds_incr[0] = 1;
     sds_incr[1] = 1;
     sds_incr[2] = 1;
     sds_incr[3] = 1;
     sds_edge[0] = 3;
     sds_edge[1] = 5;
     sds_edge[2] = 1;
     sds_edge[3] = 1;

     sds_status = read_sds(HDF_File, sds_name, sds_rank, sds_type,
                           sds_start, sds_incr, sds_edge, SW_data);

     printf ("\n\n*********************************************************\n");
     printf ("*    Example 3:  SDS                                    *\n");
     printf ("*********************************************************\n");

     for (i=0; i < 5; ++i) {
       if (i % 20 == 0) {
         fprintf(stdout,"\n\nREC#\tRegion #\tHourBox #\tSW TOA TotSky (mean)");
         fprintf(stdout,"\tSW TOA TotSky (std)\tSW TOA TotSky (num_obs)\n");
       }
       fprintf (stdout, "%4.4d:\t%8d\t%9d\t%20.6e\t%19.6e\t%23.6e\n",
                i+1, (int) Reg_data[i], (int) Hr_data[i],
                SW_data[i][0],SW_data[i][0],SW_data[i][0]);
     }

     fprintf (stdout, "\n\nCERES 32-bit Real Default    = 3.402823e+38");
     fprintf (stdout, "\nCERES 32-bit Integer Default = 2.147484e+09\n\n");

/*     for (i=0; i<5; i++)
     {
        printf ("Reg %f %f %f %f\n",Reg_data[i],Hr_data[i],
              SW_data[i][0],SW_data[i][0],SW_data[i][0]);
     } */

/* 
 ===========================================================================
 Example 4) Extract a Vdata field from the HDF file and store in memory.

  The Vdata field "ASSOCIATEDPLATFORMSHORTNAME" in Vdata
  "ASSOCIATEDPLATFORMSHORTNAME" will be extracted. 
 This example illustrates the use of functions 'get_vfield_types'
  and 'read_vdata'.

  'get_vfield_types'
     Function - Gets arrays of types and orders of the fields within a Vdata.
     Input Variables:
        HDF_File      - character pointer to the name of the HDF file.
        vdata_name    - character pointer to the name of the Vdata data set.
        num_fields    - integer defining number of fields in the Vdata.
     Output Variable:
        vfield_types  - integer array containing the types of the fields within
                        the Vdata data set (see table above).
        vfield_order  - integer array containing the order of the fields within
                        the Vdata data set.

  'read_vdata'
     Function - Reads a Vdata data set from a HDF file.
     Input Variables:
        HDF_File      - character pointer to the name of the HDF file.
        vdata_name    - character pointer to the name of the Vdata data set.
        rd_num_fields - integer defining number of fields to read.
        rd_num_recs   - integer defining number of records to read.
        vfield_name   - character pointer to the name of the vfield data set.
        vfield_types  - integer array containing the types of the fields within
                        the Vdata data set (see table above).
        vfield_order  - integer array containing the order of the fields within
                        the Vdata data set.
     Output Variable:
        vdata_data    - void pointer (cast to the array 'sat_vel' in this
                        example).

 ============================================================================
*/

     num_fields    = 14;
     vdata_name    = "CERES_metadata";
     vfield_name   = "ASSOCIATEDPLATFORMSHORTNAME";
/*
 *******************************************************************************
 * Gets arrays of types and orders of the fields within a Vdata.
 *******************************************************************************
*/
     vfield_status = get_vfield_types(HDF_File, vdata_name,
                     num_fields, vfield_types, vfield_order);
     if ( vfield_status != 0 )
     {
        fprintf (stdout,"\nError in getting Vdata field types or orders\n");
        exit(1);
     }

     vdata_data[0]=platform;
     rd_num_fields = 1;
     rd_num_recs   = 1;
/*
 *******************************************************************************
 * Read a Vdata data set from a HDF file.
 *******************************************************************************
*/
     vdata_status = read_vdata(HDF_File, vdata_name, rd_num_fields,
                    rd_num_recs, vfield_name, vfield_types, vfield_order,
                    vdata_data);

     if ( vdata_status != 0 )
     {
        fprintf (stderr,"\nError in reading Vdata: \"%s\" ", vdata_name);
        exit(1);
     }
/*
 *******************************************************************************
 * Print the value of the "ASSOCIATEDPLATFORMSHORTNAME" vfield, stored in the
 * "platform" array.
 *  Print each character ( ie,order ) in the vfield separately.  The order is
 *  also used ( 32, in this example ) to dimension the size of the array at the
 *  beginning of the code.
 *******************************************************************************
*/
     printf ("\n*********************************************************\n");
     printf ("*    Example 4: CERES_Metadata                          *\n");
     printf ("*********************************************************\n");
     printf ("\nASSOCIATEDPLATFORMSHORTNAME = ");
     for(i=0; i<32; i++)
     {
       if(platform[i] == NULL)break;
         printf("%c",platform[i]);
     }
   
     printf( "\n\n *** END OF PROGRAM ***\n\n");
     
} 












































































































